import click
import mycloudhome.configure as configure
import mycloudhome.authorization as auth
import mycloudhome.device as device
import mycloudhome.filemanagement as filemanagement
import mycloudhome.resumablefiles as resumablefiles
import mycloudhome.utils as utils


@click.group()
@click.option('--device', help='WD mycloudhome device id, can leave empty')
@click.option('--debug', is_flag=True, default=False, help='Show debug log')
def cli(device, debug):
    configure.debug = debug
    configure.device = device
    pass


@cli.command()
@click.option('--username', prompt=True, help='wd mycloudhome username')
@click.option('--password', prompt=True, hide_input=True)
def login(username, password):
    """GET WD MY CLOUD HOME ACCESS TOKEN."""
    click.echo('login...')
    configure.fetch_endpoint()
    click.echo(utils.pretty(auth.login(username, password)))


@cli.command()
def devices():
    """Get the Devices that User is attached to."""
    click.echo(utils.pretty(device.get_list_by_user_id(configure.get(
        configure.get_default_profile(), 'user_id'))))


def if_wd_path(ctx, param, value):
    # print(ctx, param, value)
    if utils.is_wd_path(value):
        return value
    raise click.BadParameter('{} is not a wd path'.format(value))


def if_local_dir(ctx, param, value):
    # print(ctx, param, value)
    if utils.is_local_dir(value):
        return value
    raise click.BadParameter('{} is not exist or not a dir'.format(value))

def if_local_file(ctx, param, value):
    # print(ctx, param, value)
    if utils.is_local_file(value):
        return value
    raise click.BadParameter('{} is not exist or is a dir'.format(value))

@cli.command()
@click.option('--wduri', prompt=True, callback=if_wd_path, help='Search files by parent directory. wd:// is the root')
@click.option('--pagetoken', default="", help='The pageToken obtained from the last response.')
def ls(wduri, pagetoken):
    """List WD objects under a prefix or under the root."""
    """<WDUri>"""
    click.echo(utils.pretty(
        filemanagement.get_file_list(wduri, pagetoken)))


@cli.command()
@click.option('--src', prompt=True, callback=if_wd_path, help='source file path in wd uri format')
@click.option('--dst', prompt=True, callback=if_wd_path, help='target file path in wd uri format')
def mv(src, dst):
    """Moves a WD object to another location in WD MY CLOUD HOME."""
    """<WDUri> <WDUri>"""
    filemanagement.move(src, dst)


@cli.command()
@click.option('--wduri', prompt=True, callback=if_wd_path, help='target file path in wd uri format')
@click.option('--name', prompt=True, help='new name')
def rename(wduri, name):
    """Rename a WD object."""
    """<WDUri> name"""
    filemanagement.rename(wduri, name)


@cli.command()
@click.option('--wduri', prompt=True, callback=if_wd_path, help='target file path in wd uri format')
def rm(wduri):
    """Deletes an WD object."""
    """<WDUri>"""
    filemanagement.delete(wduri)


@cli.command()
@click.option('--wduri', prompt=True, callback=if_wd_path, help='target file path in wd uri format')
@click.option('--localpath', prompt=True, callback=if_local_dir, help='target file path in wd uri format')
def download(wduri, localpath):
    """Copies a WD object to location locally."""
    """<WDUri> <LocalPath>"""
    filemanagement.download(wduri, localpath)


@cli.command()
@click.option('--wduri', prompt=True, callback=if_wd_path, help='target file path in wd uri format')
def mkdir(wduri):
    """Create an WD dir."""
    """<WDUri>"""
    filemanagement.mkdir(wduri)


@cli.command()
@click.option('--localpath', prompt=True, callback=if_local_file, help='target file path in wd uri format')
@click.option('--wduri', prompt=True, callback=if_wd_path, help='target file path in wd uri format')
def upload(localpath,wduri):
    """Copies a local file to location in WD MY CLOUD HOME."""
    """<LocalPath> <WDUri>"""
    resumablefiles.upload(localpath,wduri)


# TODO
# @cli.command()
# def sync():
#     pass

if __name__ == '__main__':
    cli()



import json
import os

import requests

import mycloudhome.configure as configure
import mycloudhome.authorization as auth
import mycloudhome.utils as utils


def upload(localPath, wdUri):
    dir_id = utils.convert_wd_path_to_wd_id(wdUri)
    file_name = utils.get_local_file_name(localPath)

    localtion = create_resumable_file(file_name, dir_id)

    url = configure.get(configure.get_default_profile(), 'externaluri') + localtion + '/resumable/content'
    # files = {'file': open(localPath, 'rb')}
    files = {'file': (file_name, open(localPath, 'rb'), 'application/x-www-form-urlencoded')}

    response = requests.put(url, files=files, headers={'Authorization': 'Bearer '+auth.token()})

    if configure.debug:
        print(url)
        print(response)
        print(response.content)

    mark_resumable_file_as_done(localtion)


def create_resumable_file(name, parentID):
    url = configure.get(configure.get_default_profile(),
                        'externaluri') + '/sdk/v2/files?resolveNameConflict=1'

    payload = """
--foo

{{"name":"{}","parentID":"{}"}}
--foo

--foo--
""".format(name, parentID)

    response = requests.post(url,
                             data=payload,
                             headers={
                                 "Content-Type": "multipart/related; boundary=foo",
                                 'Authorization': 'Bearer '+auth.token()
                             }
                             )
    if configure.debug:
        print(url)
        print(payload)
        print(response)
        print(response.content)
        print(response.headers)
    if response.status_code != 201:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))

    return response.headers['Location']


def mark_resumable_file_as_done(fileLocation):
    url = configure.get(configure.get_default_profile(),
                        'externaluri') + fileLocation + '/resumable/content?done=true'

    response = requests.put(
        url, headers={'Authorization': 'Bearer '+auth.token()})

    if configure.debug:
        print(url)
        print(response)
        print(response.content)



import json
import os

import requests

import mycloudhome.configure as configure
import mycloudhome.authorization as auth
import mycloudhome.utils as utils

from pprint import pformat

def get_list_by_user_id(user_id):
    url = configure.get('cloud.service.urls',
                        'service.device.url')+'/device/v1/user/{}?pretty=true'
    
    response = requests.get(url.format(user_id), headers={
                            'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url.format(user_id))                        
        print(response.content)

    if response.status_code != 200:
        raise Exception("DEVICE INFO Error !!! [{}][{}]".format(
            response.status_code, response.content))

    device_info = json.loads(response.content)
    return device_info['data']


def get_by_device_id(device_id):
    url = configure.get('cloud.service.urls',
                        'service.device.url')+'/device/v1/device/{}?pretty=true'
    
    response = requests.get(url.format(device_id), headers={
                            'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url.format(device_id))
        print(response.content)

    if response.status_code != 200:
        raise Exception("DEVICE INFO Error !!! [{}][{}]".format(
            response.status_code, response.content))

    device_info = json.loads(response.content)
    return device_info['data']


def get_user_list_by_device_id(device_id):
    url = configure.get('cloud.service.urls',
                        'service.device.url')+'/device/v1/device/{}/user?pretty=true'

    response = requests.get(url.format(device_id), headers={
                            'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url.format(device_id))
        print(response.content)
        
    if response.status_code != 200:
        raise Exception("DEVICE INFO Error !!! [{}][{}]".format(
            response.status_code, response.content))

    device_info = json.loads(response.content)
    return device_info['data']



import json
import os
import configparser
import requests

home = os.path.expanduser('~')

debug = False
device = None


def fetch_endpoint():
    response = requests.get("https://config.mycloud.com/config/v1/config")

    if response.status_code != 200:
        raise Exception("Endpoint Error !!! [{}][{}]".format(
            response.status_code, response.content))

    config = json.loads(response.content)['data']
    # print(config)
    write('cloud.service.urls', config['componentMap']['cloud.service.urls'])


def get(section, key):
    config = configparser.ConfigParser()
    config.read(os.path.join(home, 'mycloudhome.ini'))
    return config[section][key]

# def save(section, key, value):
#     config = configparser.ConfigParser()
#     config.read(os.path.join(home, 'mycloudhome.ini'))
#     config['section'][key] = value

#     with open(os.path.join(home, 'mycloudhome.ini'), 'w') as configfile:
#         config.write(configfile)


def save_token(token):
    config = configparser.ConfigParser()
    config.read(os.path.join(home, 'mycloudhome.ini'))
    config['auth'] = token

    with open(os.path.join(home, 'mycloudhome.ini'), 'w') as configfile:
        config.write(configfile)


def get_token():
    config = configparser.ConfigParser()
    config.read(os.path.join(home, 'mycloudhome.ini'))
    return config['auth']['id_token']


def save_profiles(devices, user):
    config = configparser.ConfigParser()
    config.read(os.path.join(home, 'mycloudhome.ini'))

    for device in devices:
        profile_name = device['deviceId']
        config[profile_name] = device['network']
        config[profile_name]['deviceId'] = device['deviceId']
        config[profile_name]['serialNumber'] = device['serialNumber']
        config[profile_name]['name'] = device['name']
        config[profile_name]['email'] = user['email']
        config[profile_name]['name'] = user['name']
        config[profile_name]['user_id'] = user['user_id']
        config[profile_name]['first_name'] = user['user_metadata']['first_name']
        config[profile_name]['last_name'] = user['user_metadata']['last_name']
        config[profile_name]['lang'] = user['user_metadata']['lang']
        config[profile_name]['time_zone_name'] = user['user_metadata']['time_zone_name']

    with open(os.path.join(home, 'mycloudhome.ini'), 'w') as configfile:
        config.write(configfile)


def get_default_profile():
    config = configparser.ConfigParser()
    config.read(os.path.join(home, 'mycloudhome.ini'))
    sections = config.sections()
    sections.remove('cloud.service.urls')
    sections.remove('auth')

    if device is not None and device in sections:
        return device

    if os.getenv('WD_DEVICE') and os.getenv('WD_DEVICE') in sections:
        return os.getenv('WD_DEVICE')

    return sections[0]


def write(section, configuration):
    config = configparser.ConfigParser()
    config.read(os.path.join(home, 'mycloudhome.ini'))
    config[section] = configuration

    with open(os.path.join(home, 'mycloudhome.ini'), 'w') as configfile:
        config.write(configfile)



import json
import os

import requests
from progress.bar import Bar

import mycloudhome.configure as configure
import mycloudhome.authorization as auth
import mycloudhome.utils as utils


def get_file_list(wdUri, pagetoken=""):
    if not utils.is_wd_path(wdUri):
        raise Exception("Not WD PATH !!!")
    dir_id = utils.convert_wd_path_to_wd_id(wdUri)
    info = get_file_by_id(dir_id)
    if info['mimeType'] == "application/x.wd.dir":
        return get_file_list_by_id(dir_id, pagetoken)
    else:
        return info


def get_file_list_by_id(wdUri, pagetoken=""):
    url = configure.get(configure.get_default_profile(), 'externaluri') + \
        '/sdk/v2/filesSearch/parents?ids={}&limit=100&order=desc&orderBy=name&pretty=true&pageToken={}'

    response = requests.get(url.format(wdUri, pagetoken), headers={
                            'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url.format(wdUri, pagetoken))
        print(response.content)

    if response.status_code != 200:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))

    file_info = json.loads(response.content)
    return file_info


def get_file_by_id(file_id):
    url = configure.get(configure.get_default_profile(), 'externaluri') + \
        '/sdk/v2/files/{}?pretty=true'

    response = requests.get(url.format(file_id), headers={
                            'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url.format(file_id))
        print(response.content)

    if response.status_code != 200:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))

    file_info = json.loads(response.content)
    return file_info


def get_file_under_dir_by_name(name, dir_id, pagetoken=""):
    url = configure.get(configure.get_default_profile(), 'externaluri') + \
        '/sdk/v2/filesSearch/parents?ids={}&limit=100&order=desc&orderBy=name&pretty=true&pageToken={}'

    response = requests.get(url.format(dir_id, pagetoken), headers={
                            'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print("search {} in parents {}".format(name, dir_id))
        print(url.format(dir_id, pagetoken))
        print(response.content)

    if response.status_code != 200:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))

    data = json.loads(response.content)

    if 'files' in data:
        for item in data['files']:
            if item['name'] == name:
                return item
        if data['pageToken'] != "":
            return get_file_under_dir_by_name(name, dir_id, data['pageToken'])

    return None


def move(src, dst):
    src_id = utils.convert_wd_path_to_wd_id(src)
    dst_id = utils.convert_wd_path_to_wd_id(dst)

    url = configure.get(configure.get_default_profile(), 'externaluri') + \
        '/sdk/v2/files/{}/patch'.format(src_id)
    payload = {"parentID": dst_id}

    response = requests.post(url, data=json.dumps(payload), headers={
                             "Content-Type": "application/json", 'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url)
        print(payload)
        print(response.content)
    if response.status_code != 204:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))


def delete(wdUri):
    file_id = utils.convert_wd_path_to_wd_id(wdUri)
    url = configure.get(configure.get_default_profile(), 'externaluri') + \
        '/sdk/v2/files/{}'.format(file_id)

    response = requests.delete(url, headers={
                               "Content-Type": "application/json", 'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url)
        print(response.content)
    if response.status_code in [200, 202, 204]:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))


def rename(wdUri, name):
    file_id = utils.convert_wd_path_to_wd_id(wdUri)
    url = configure.get(configure.get_default_profile(), 'externaluri') + \
        '/sdk/v2/files/{}/patch'.format(file_id)
    payload = {"name": name}

    response = requests.post(url, data=json.dumps(payload), headers={
                             "Content-Type": "application/json", 'Authorization': 'Bearer '+auth.token()})
    if configure.debug:
        print(url)
        print(payload)
        print(response.content)
    if response.status_code != 204:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))


def download(wdUri, localPath):
    file_id = utils.convert_wd_path_to_wd_id(wdUri)
    url = configure.get(configure.get_default_profile(), 'externaluri') + \
        '/sdk/v3/files/{}/content'.format(file_id)

    response = requests.get(url,
                            headers={"Content-Type": "application/json",
                                     'Authorization': 'Bearer '+auth.token()},
                            stream=True
                            )

    fileName = os.path.join(localPath, get_file_by_id(file_id)['name'])
    if configure.debug:
        print(url)
        print(fileName)

    if response.status_code != 200:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))

    progress = Bar('Downloading', max=get_file_by_id(
        file_id)['size'], suffix='%(percent)d%%')

    with open(fileName, 'wb') as file_handler:
        for chunk in response.iter_content(1024):
            file_handler.write(chunk)
            progress.next(1024)
        progress.finish()


def mkdir(wdUri):
    idx = 'root'
    for name in wdUri.replace("wd://", "").split('/'):
        if name == "":
            continue

        sub_dir = get_file_under_dir_by_name(name, idx)
        if sub_dir is None:
            create_floder(name, idx)
            continue

        if sub_dir['mimeType'] != "application/x.wd.dir":
            raise Exception("WD PATH IS NOT DIR!!! [{}]".format(name))

        idx = sub_dir['id']


def create_floder(name, parentID):
    url = configure.get(configure.get_default_profile(),
                        'externaluri') + '/sdk/v2/files?resolveNameConflict=1'

    payload = """
--xxoo1707a7a80f1xxoo

{{"name":"{}","parentID":"{}","mimeType":"application/x.wd.dir"}}
--xxoo1707a7a80f1xxoo--
""".format(name, parentID)

    response = requests.post(url,
                             data=payload,
                             headers={
                                 "Content-Type": "multipart/related; boundary=xxoo1707a7a80f1xxoo",
                                 'Authorization': 'Bearer '+auth.token()
                             }
                             )
    if configure.debug:
        print(url)
        print(payload)
        print(response.content)
    if response.status_code != 201:
        raise Exception("FILE MANAGEMENT Error !!! [{}][{}]".format(
            response.status_code, response.content))
